package ch.unige.solidify;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;

import org.apache.maven.plugin.logging.Log;

import ch.unige.solidify.model.xml.xhtml.v1.XhtmlHtmlType;
import ch.unige.solidify.model.xml.xhtml.v1.XhtmlUlType;
import ch.unige.solidify.process.HrefUpdateProcess;
import ch.unige.solidify.process.TocExtractionException;
import ch.unige.solidify.process.TocExtractionProcess;

public class AsciidoctorTocExtracter {
  private Path file;
  private JAXBContext jaxbContext;
  JAXBElement<XhtmlHtmlType> html;
  JAXBElement<XhtmlUlType> toc;

  public AsciidoctorTocExtracter() {
  }

  public void load(String inputFilePath) throws JAXBException, IOException {
    this.jaxbContext = JAXBContext.newInstance(XhtmlHtmlType.class, XhtmlUlType.class);
    this.file = Paths.get(inputFilePath);
    this.html = this.loadHtml(this.file);
  }

  public void run(Log log) throws TocExtractionException {
    // ******************************
    // ** !! Order is important !! **
    // ******************************
    // Extract TOC
    this.log(log, "Extracting TOC from HTML document");
    this.toc = new TocExtractionProcess().process(this.html, this.toc);
    // Update links
    this.log(log, "Updating links in HTML document");
    this.toc = new HrefUpdateProcess(this.file.getFileName().toString()).process(this.html, this.toc);
  }

  public void generate() throws JAXBException, IOException {
    this.saveHtml(this.getTocName(this.file));
  }

  private void log(Log log, String message) {
    if (log == null) {
      System.out.println(message);
    } else {
      log.info(message);
    }
  }

  private Path getTocName(Path file) {
    return Paths.get(this.getNewName(file, "toc"));
  }

  private String getNewName(Path file, String suffix) {
    return file.getParent().toString() + File.separatorChar
            + file.getFileName().toString().replaceFirst(".html", "-" + suffix + ".html");
  }

  private JAXBElement<XhtmlHtmlType> loadHtml(Path file) throws IOException, JAXBException {
    JAXBElement<XhtmlHtmlType> root = null;
    // Load HTML Document
    try (FileReader fileReader = new FileReader(file.toFile())) {
      root = (JAXBElement<XhtmlHtmlType>) this.jaxbContext.createUnmarshaller().unmarshal(fileReader);
    } catch (IOException | JAXBException e) {
      throw e;
    }
    return root;
  }

  private void saveHtml(Path outputFile) throws JAXBException, IOException {
    try (FileWriter fileWriter = new FileWriter(outputFile.toFile())) {
      this.jaxbContext.createMarshaller().marshal(this.toc, fileWriter);
    } catch (JAXBException | IOException e) {
      throw e;
    }
  }
}
