package ch.unige.solidify;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.bind.JAXBException;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;

import ch.unige.solidify.process.TocExtractionException;

@Mojo(name = "extract-toc", defaultPhase = LifecyclePhase.PREPARE_PACKAGE)
public class AsciidoctorTocExtracterMojo extends AbstractMojo {

  @Parameter(property = "inputFile", required = true)
  private String inputFile;

  @Parameter(property = "skip", required = false, defaultValue = "false")
  private boolean skip;

  @SuppressWarnings("squid:S1148")
  public void execute() throws MojoExecutionException {
    if (skip) {
      getLog().info("Skipping asciidoctor TOC extraction");
      return;
    }
    AsciidoctorTocExtracter tocExtractor = new AsciidoctorTocExtracter();
    try {
      getLog().info("Check input file: " + this.inputFile);
      tocExtractor.load(inputFile);
      getLog().info("Process asciidoctor TOC extraction");
      tocExtractor.run(getLog());
      tocExtractor.generate();
      getLog().info("Successfully HTML TOC generated");
    } catch (FileNotFoundException e) {
      getLog().info("File not found: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    } catch (IOException e) {
      getLog().info("IO Exception: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    } catch (JAXBException e) {
      getLog().info("JAXB Exception: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    } catch (TocExtractionException e) {
      getLog().info("TOC Extraction Exception: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    }

  }
}
