package ch.unige.solidify.process;

import java.io.Serializable;
import java.util.List;

import javax.xml.bind.JAXBElement;

import ch.unige.solidify.model.xml.xhtml.v1.XhtmlDivType;
import ch.unige.solidify.model.xml.xhtml.v1.XhtmlHtmlType;
import ch.unige.solidify.model.xml.xhtml.v1.XhtmlUlType;

public class TocExtractionProcess extends BaseProcess {

  public TocExtractionProcess() {
  }

  @Override
  public JAXBElement<XhtmlUlType> process(JAXBElement<XhtmlHtmlType> html, JAXBElement<XhtmlUlType> toc) throws TocExtractionException {
    // Extract TOC by removing content
    if (html.getValue() == null) {
      throw new TocExtractionException("Cannot load HTML document");
    }
    if (html.getValue().getBody() == null) {
      throw new TocExtractionException("Cannot load HTML body");
    }
    if (html.getValue().getBody().getXhtmlBlockMix() == null) {
      throw new TocExtractionException("Cannot load HTML body blocks");
    }
    for (Serializable item : html.getValue().getBody().getXhtmlBlockMix()) {
      if (item instanceof XhtmlDivType) {
        XhtmlDivType div = (XhtmlDivType) item;
        if (div.getId().equals("header")) {
          return this.processHeader(div.getContent());
        }
      }
    }
    throw new TocExtractionException("Cannot find HTML header");
  }

  private JAXBElement<XhtmlUlType> processHeader(List<Serializable> content) throws TocExtractionException {
    for (Serializable item : content) {
      if (item instanceof JAXBElement<?>) {
        Object obj = ((JAXBElement<?>) item).getValue();
        if (obj instanceof XhtmlDivType) {
          XhtmlDivType div = (XhtmlDivType) obj;
          if (div.getId().equals("toc")) {
            return this.processToc(div.getContent());
          }
        }
      }
    }
    throw new TocExtractionException("Cannot find TOC");
  }

  private JAXBElement<XhtmlUlType> processToc(List<Serializable> content) throws TocExtractionException {
    for (Serializable item : content) {
      if (item instanceof JAXBElement<?>) {
        Object obj = ((JAXBElement<?>) item).getValue();
        if (obj instanceof XhtmlUlType) {
          XhtmlUlType ul = (XhtmlUlType) obj;
          if (ul.getClazz() != null && ul.getClazz().equals("sectlevel1")) {
            return (JAXBElement<XhtmlUlType>) item;
          }
        }
      }
    }
    throw new TocExtractionException("Cannot find TOC content");
  }

}
