package ch.unige.solidify;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;

@Mojo(name = "clean-json", defaultPhase = LifecyclePhase.PREPARE_PACKAGE)
public class OpenApiCleanerMojo extends AbstractMojo {

  @Parameter(property = "inputFile", required = true)
  private String inputFile;

  @Parameter(property = "inputExtraDocumentationFile", required = true)
  private String inputExtraDocumentationFile;

  @Parameter(property = "outputFile", required = true)
  private String outputFile;

  @Parameter(property = "contextPath", required = true)
  private String contextPath;

  @Parameter(property = "securityName", required = true)
  private String securityName;

  @Parameter(property = "modules", required = true)
  private List<String> modules;

  @Parameter(property = "skip", required = false, defaultValue = "false")
  private boolean skip;

  @SuppressWarnings("squid:S1148")
  public void execute() throws MojoExecutionException {
    if (skip) {
      getLog().info("Skipping clean of openAPI json file");
      return;
    }
    JsonCleaner jsonCleaner = new JsonCleaner();
    try {
      getLog().info(getFileReadLogMessage());
      jsonCleaner.load(inputFile, inputExtraDocumentationFile);
      getLog().info("Process JSON Cleanup");
      jsonCleaner.run(contextPath, modules, securityName, getLog());
      getLog().info("Write file in " + outputFile);
      jsonCleaner.generate(outputFile);
      getLog().info("Successfully processed JSON Object in " + outputFile);
    } catch (FileNotFoundException e) {
      e.printStackTrace();
      getLog().info("File not found: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    } catch (IOException e) {
      e.printStackTrace();
      getLog().info("IO Exception: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    }
  }

  private String getFileReadLogMessage() {
    StringBuilder messageReadFileBuilder = new StringBuilder();
    messageReadFileBuilder.append("Read openapi file : " + inputFile);
    if (inputExtraDocumentationFile == null) {
      messageReadFileBuilder.append(" and no extra documentation provided");
    } else {

      messageReadFileBuilder
              .append(" and extra documentation file : " + inputExtraDocumentationFile);
    }
    return messageReadFileBuilder.toString();
  }
}
