package ch.unige.solidify.model;

import ch.unige.solidify.process.BaseProcess;

public class ParametersObject {
  private SchemaObject schema;
  private String in;
  private String name;
  private String description;
  private boolean required;
  private String style;
  private boolean explode;

  public ParametersObject() {
    this.in = "path";
    this.name = "resId";
    this.description = "Resource Id (GUID)";
    this.required = true;
    this.schema = new SchemaObject();
    this.schema.type = "string";
  }

  public ParametersObject(String name, String in, String description, boolean required, String type) {
    this.in = in;
    this.name = name;
    this.description = description;
    this.required = required;
    this.schema = new SchemaObject();
    this.schema.type = type;
  }

  public ParametersObject(String name, String in, String description, boolean required, String type, String format) {
    this(name, in, description, required, type);
    this.schema.format = format;
  }

  public static ParametersObject getFiltersFormQueryParameters() {
    ParametersObject parametersObject = new ParametersObject();
    parametersObject.in = BaseProcess.QUERY;
    parametersObject.name = BaseProcess.FILTERS;
    parametersObject.schema.type = BaseProcess.OBJECT;
    parametersObject.schema.additionalProperties = new AdditionalPropertiesObject(BaseProcess.STRING);
    parametersObject.description = "Filters forms to add where condition. To filter on a field if the field is embedded in a sub structure, the field name must be fully named with “.” for each level:+ <sub structure name>.<field name>";
    parametersObject.style = BaseProcess.FORM;
    parametersObject.required = false;
    parametersObject.explode = true;
    return parametersObject;
  }

  public class SchemaObject {
    private String type;
    private String format;
    private AdditionalPropertiesObject additionalProperties;

    private SchemaObject() {
    }

    public String getType() {
      return type;
    }

    public void setType(String type) {
      this.type = type;
    }

    public String getFormat() {
      return format;
    }

    public void setFormat(String format) {
      this.format = format;
    }

    public AdditionalPropertiesObject getAdditionalProperties() {
      return additionalProperties;
    }

    public void setAdditionalProperties(AdditionalPropertiesObject additionalProperties) {
      this.additionalProperties = additionalProperties;
    }

  }

  public SchemaObject getSchema() {
    return schema;
  }

  public String getIn() {
    return in;
  }

  public String getName() {
    return name;
  }

  public String getDescription() {
    return description;
  }

  public boolean isRequired() {
    return required;
  }

  public String getStyle() {
    return style;
  }

  public boolean isExplode() {
    return explode;
  }

}
