package ch.unige.solidify.process;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.json.JSONObject;

import ch.unige.solidify.model.OpenApiExtraDocumentation;

/**
 * Define old & new name relationship with webservice
 */
public class SchemaNameDefinitionProcess extends BaseProcess {
  private Map<String, String> migrationNameMap;
  private List<OpenApiExtraDocumentation> listOpenApiExtraDocumentation;
  private ArrayList<String> moduleList = new ArrayList<>();

  public SchemaNameDefinitionProcess(List<String> modules, Map<String, String> migrationNameMap,
          String jsonExtraDocumentation) {
    this.setModules(modules);
    this.migrationNameMap = migrationNameMap;
    this.listOpenApiExtraDocumentation = this.loadOpenApiExtraDocumentation(jsonExtraDocumentation);
  }

  public void process(JSONObject json) {
    // Change schema name for application
    JSONObject applicaionJson = this.getPathsNode(json).getJSONObject("/");
    updateSchemaInPath(applicaionJson, "application");
    // Change schema name every modules
    for (String module : this.getModules()) {
      JSONObject moduleJson = this.getPathsNode(json).getJSONObject(module);
      updateSchemaInPath(moduleJson, module.substring(1));
    }
    // Change schema name every web services
    for (OpenApiExtraDocumentation resource : listOpenApiExtraDocumentation) {
      JSONObject resourceJson = this.getPathsNode(json).getJSONObject(resource.getPath());
      updateSchemaInPath(resourceJson, resource.getSchema());
    }
    // Migrate schemas with new name
    migrateSchemaNames(json);
  }

  private ArrayList<String> getModules() {
    return this.moduleList;
  }

  private void setModules(List<String> modules) {
    for (String module : modules) {
      if (module.startsWith("/")) {
        this.moduleList.add(module);
      } else {
        this.moduleList.add("/" + module);
      }

    }
  }

  private void migrateSchemaNames(JSONObject json) {
    JSONObject schemas = this.getSchemasNode(json);
    for (Map.Entry<String, String> names : this.migrationNameMap.entrySet()) {
      // Get JSON values
      Object obj = schemas.get(names.getKey());
      // Add JSON values with new name
      schemas.put(names.getValue(), obj);
      // Remove JSON values with old name
      schemas.remove(names.getKey());
    }
  }

  private void updateSchemaInPath(JSONObject resource, String newSchemaName) {
    // Iterate for each web service
    for (String httpVerb : this.getHttpVerb()) {
      if (!resource.isNull(httpVerb)) {
        JSONObject ws = resource.getJSONObject(httpVerb);

        String originalModelName = ws.getJSONObject("responses")
                .getJSONObject(this.getResponse(httpVerb)).getJSONObject("content")
                .getJSONObject("application/hal+json").getJSONObject("schema").getString("$ref")
                .substring(REF_SCHEMAS_VALUE.length());
        migrationNameMap.put(originalModelName, newSchemaName);
        updateSchemaReferences(ws, httpVerb, newSchemaName);
      }
    }
  }

  private void updateSchemaReferences(JSONObject ws, String httpVerb, String newSchemaName) {
    if (!ws.isNull("requestBody")) {
      ws.getJSONObject("requestBody").getJSONObject("content").getJSONObject("application/json")
              .getJSONObject("schema").put("$ref", REF_SCHEMAS_VALUE + newSchemaName);
    }
    ws.getJSONObject("responses").getJSONObject(this.getResponse(httpVerb)).getJSONObject("content")
            .getJSONObject("application/hal+json").getJSONObject("schema")
            .put("$ref", REF_SCHEMAS_VALUE + newSchemaName);
  }

}
