package ch.unige.solidify.test;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

import org.json.JSONObject;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;

import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

import ch.unige.solidify.JsonCleaner;

@ExtendWith(MockitoExtension.class)
public class JsonCleanerTest {
  private final String SRC_TEST_RESOURCES = "src/test/resources/";
  private final String JSON = ".json";
  private final String FILE_OPEN_API_TO_TEST = "openapi-3.0";
  private final String FILE_OPEN_API_EXTRA_DOCUMENTATION_TO_TEST = "openapi-extra-documentation";
  private final String CONTEXT_PATH = "/dlcm";
  private final String SECURITY_NAME = "dlcm-auth";
  private final List<String> MODULE_LIST = Arrays
          .asList("/admin", "/access", "/archival-storage", "/data-mgmt", "/ingest", "/preingest", "/resource-srv",
                  "/preservation-planning");

  private final JsonCleaner jsonCleaner = new JsonCleaner();

  @Test
  public void testReadFile() {
    readFile();

    assertNotNull(jsonCleaner.getJson());
  }

  @Test
  public void testRun() {
    readFile();

    JSONObject jsonBeforeCleanup = copyJsonObject(jsonCleaner.getJson());
    jsonCleaner.run(CONTEXT_PATH, MODULE_LIST, SECURITY_NAME, null);
    JSONObject jsonAfterCleanup = jsonCleaner.getJson();

    new AssertGeneralProcessImpl(jsonBeforeCleanup).process(jsonAfterCleanup);
    new AssertSchemasProcessImpl(jsonBeforeCleanup).process(jsonAfterCleanup);
    new AssertPathsProcessImpl(jsonBeforeCleanup).process(jsonAfterCleanup);

    try {
      jsonCleaner.generate("target/output/insideSubDir/" + FILE_OPEN_API_TO_TEST + "-cleaned.json");
    } catch (IOException e) {
      fail("IO exception: " + e.getMessage());
      e.printStackTrace();
    }
  }

  @Test
  public void sortJSON() throws IOException {
    readFile();
    String jsonString = jsonCleaner.getJson().toString();

    ObjectMapper mapper = new ObjectMapper().copy()
            .configure(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS, true)
            .configure(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY, true)
            .configure(SerializationFeature.INDENT_OUTPUT, true);
    Object json = mapper.readValue(jsonString, Object.class);
    saveData(mapper.writeValueAsString(json));
  }

  private void saveData(String string) throws IOException {
    // Get the file reference
    Path json = Paths.get("target/output/insideSubDir/" + "openapi-3.0-sorted.json");

    // Use try-with-resource to get auto-closeable writer instance
    try (BufferedWriter writer = Files.newBufferedWriter(json)) {
      writer.write(string);
    }
  }

  private JSONObject copyJsonObject(JSONObject jsonObjectToCopy) {
    return new JSONObject(jsonObjectToCopy.toString());
  }

  private void readFile() {
    try {
      jsonCleaner.load(SRC_TEST_RESOURCES + FILE_OPEN_API_TO_TEST + JSON,
              SRC_TEST_RESOURCES + FILE_OPEN_API_EXTRA_DOCUMENTATION_TO_TEST + JSON);
    } catch (IOException e) {
      fail("Test file not found: " + e.getMessage());
      e.printStackTrace();
    }
  }
}
